@@ -1,6 +1,7 @@
 import abc
 import os
 import re
+import sys
 
 import whoosh
 import whoosh.fields
@@ -11,8 +12,21 @@ from flask.ext.sqlalchemy import models_committed, BaseQuery
 from sqlalchemy.orm.mapper import Mapper
 
 class WhoosheeQuery(BaseQuery):
+    """An override for SQLAlchemy query used to do fulltext search."""
+
     # TODO: add an option to override used Whoosheer
     def whooshee_search(self, search_string, group=whoosh.qparser.OrGroup, match_substrings=True):
+        """Do a fulltext search on the query.
+
+        Args:
+            search_string: string to search for
+            group: whoosh group to use for searching, defaults to OrGroup (searches for all
+                   words in all columns)
+            match_substrings: True if you want to match substrings, False otherwise
+
+        Returns:
+            query filtered with results of the fulltext search
+        """
         ### inspiration taken from flask-WhooshAlchemy
         # find out all entities in join
         entities = set()
@@ -22,7 +36,7 @@ class WhoosheeQuery(BaseQuery):
         # joined entities
         if self._join_entities and isinstance(self._join_entities[0], Mapper):
             # SQLAlchemy >= 0.8.0
-            entities.update(set(map(lambda x: x.entity, self._join_entities)))
+            entities.update(set([x.entity for x in self._join_entities]))
         else:
             # SQLAlchemy < 0.8.0
             entities.update(set(self._join_entities))
@@ -30,7 +44,7 @@ class WhoosheeQuery(BaseQuery):
         whoosheer = next(w for w in Whooshee.whoosheers if set(w.models) == entities)
 
         # TODO what if unique field doesn't exist or there are multiple?
-        for fname, field in whoosheer.schema._fields.items():
+        for fname, field in list(whoosheer.schema._fields.items()):
             if field.unique:
                 uniq = fname
 
@@ -57,21 +71,40 @@ class WhoosheeQuery(BaseQuery):
         return self.filter(attr.in_(res))
 
 class AbstractWhoosheer(object):
-    __metaclass__ = abc.ABCMeta
+    """A superclass for all whoosheers.
+
+    Whoosheer is basically a unit of fulltext search. It represents either of:
+
+    * One table, in which case all given fields of the model is searched.
+    * More tables, in which case all given fields of all the tables are searched.
+    """
 
     @classmethod
     def search(cls, search_string, values_of='', group=whoosh.qparser.OrGroup, match_substrings=True):
+        """Actually searches the fields for given search_string.
+
+        Args:
+            search_string: string to search for
+            values_of: if given, the method will not return the whole records, but only values
+                       of given column (defaults to returning whole records)
+            group: whoosh group to use for searching, defaults to OrGroup (searches for all
+                   words in all columns)
+            match_substrings: True if you want to match substrings, False otherwise
+        Returns:
+            Found records if 'not values_of', else values of given column
+        """
         prepped_string = cls.prep_search_string(search_string, match_substrings)
         with cls.index.searcher() as searcher:
             parser = whoosh.qparser.MultifieldParser(cls.schema.names(), cls.index.schema, group=group)
             query = parser.parse(prepped_string)
             results = searcher.search(query)
             if values_of:
-                return map(lambda x: x[values_of], results)
+                return [x[values_of] for x in results]
             return results
 
     @classmethod
     def prep_search_string(cls, search_string, match_substrings):
+        """Prepares search string as a proper whoosh search string."""
         s = search_string.strip()
         # we don't want stars from user
         s = s.replace('*', '')
@@ -83,7 +116,12 @@ class AbstractWhoosheer(object):
         # TODO: some sanitization
         return s
 
+AbstractWhoosheerMeta = abc.ABCMeta('AbstractWhoosheer', (AbstractWhoosheer,), {})
+
 class Whooshee(object):
+    """A top level class that allows to register whoosheers and adds an on_commit hook
+    to SQLAlchemy."""
+
     _underscore_re1 = re.compile(r'(.)([A-Z][a-z]+)')
     _underscore_re2 = re.compile('([a-z0-9])([A-Z])')
     whoosheers = []
@@ -91,11 +129,18 @@ class Whooshee(object):
     def __init__(self, app):
         self.index_path_root = app.config.get('WHOOSHEE_DIR', '') or 'whooshee'
         self.search_string_min_len = app.config.get('WHOSHEE_MIN_STRING_LEN', 3)
+        self.writer_timeout = app.config.get('WHOOSHEE_WRITER_TIMEOUT', 2)
         models_committed.connect(self.on_commit, sender=app)
         if not os.path.exists(self.index_path_root):
             os.makedirs(self.index_path_root)
 
     def register_whoosheer(self, wh):
+        """Registers a given whoosheer:
+
+        * Creates and opens an index for it (if it doesn't exist yet)
+        * Sets some default values on it (unless they're already set)
+        * Replaces query class of every whoosheer's model by WhoosheeQuery
+        """
         if not hasattr(wh, 'search_string_min_len'):
             wh.search_string_min_len = self.search_string_min_len
         if not hasattr(wh, 'index_subdir'):
@@ -108,9 +153,12 @@ class Whooshee(object):
             model.query_class = WhoosheeQuery
         return wh
 
-    def register_model(self, *index_fields):
+    def register_model(self, *index_fields, **kw):
+        """Registers a single model for fulltext search. This basically creates
+        a simple Whoosheer for the model and calls self.register_whoosheer on it.
+        """
         # construct subclass of AbstractWhoosheer for a model
-        class ModelWhoosheer(AbstractWhoosheer):
+        class ModelWhoosheer(AbstractWhoosheerMeta):
             pass
 
         mwh = ModelWhoosheer
@@ -125,7 +173,7 @@ class Whooshee(object):
                     primary = field.name
                     schema_attrs[field.name] = whoosh.fields.NUMERIC(stored=True, unique=True)
                 elif field.name in index_fields:
-                    schema_attrs[field.name] = whoosh.fields.TEXT()
+                    schema_attrs[field.name] = whoosh.fields.TEXT(**kw)
             mwh.schema = whoosh.fields.Schema(**schema_attrs)
             # we can't check with isinstance, because ModelWhoosheer is private
             # so use this attribute to find out
@@ -137,7 +185,10 @@ class Whooshee(object):
                 for f in index_fields:
                     attrs[f] = getattr(model, f)
                     if not isinstance(attrs[f], int):
-                        attrs[f] = unicode(attrs[f])
+                        if sys.version < '3':
+                            attrs[f] = unicode(attrs[f])
+                        else:
+                            attrs[f] = str(attrs[f])
                 writer.update_document(**attrs)
 
             @classmethod
@@ -146,7 +197,10 @@ class Whooshee(object):
                 for f in index_fields:
                     attrs[f] = getattr(model, f)
                     if not isinstance(attrs[f], int):
-                        attrs[f] = unicode(attrs[f])
+                        if sys.version < '3':
+                            attrs[f] = unicode(attrs[f])
+                        else:
+                            attrs[f] = str(attrs[f])
                 writer.add_document(**attrs)
 
             setattr(mwh, 'update_{0}'.format(model.__name__.lower()), update_model)
@@ -160,6 +214,10 @@ class Whooshee(object):
         return inner
 
     def create_index(self, wh):
+        """Creates and opens index for given whoosheer.
+
+        If the index already exists, it just opens it, otherwise it creates it first.
+        """
         index_path = os.path.join(self.index_path_root, wh.index_subdir)
         if whoosh.index.exists_in(index_path):
             index = whoosh.index.open_dir(index_path)
@@ -170,8 +228,11 @@ class Whooshee(object):
         wh.index = index
 
     def on_commit(self, app, changes):
+        """Method that gets connected to flask.ext.sqlalchemy.models_committed, where it serves
+        to do the actual index writing.
+        """
         for wh in self.__class__.whoosheers:
-            writer = wh.index.writer()
+            writer = wh.index.writer(timeout=self.writer_timeout)
             for change in changes:
                 if change[0].__class__ in wh.models:
                     method_name = '{0}_{1}'.format(change[1], change[0].__class__.__name__.lower())
@@ -179,4 +240,5 @@ class Whooshee(object):
             writer.commit()
 
     def camel_to_snake(self, s):
+        """Constructs nice dir name from class name, e.g. FooBar => foo_bar."""
         return self._underscore_re2.sub(r'\1_\2', self._underscore_re1.sub(r'\1_\2', s)).lower()
